package com.fossgalaxy.games.tbs.metrics;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import com.fossgalaxy.games.tbs.io.SettingsIO;
import com.fossgalaxy.games.tbs.metrics.parameters.ParameterList;
import com.fossgalaxy.games.tbs.metrics.parameters.Parameter;
import com.fossgalaxy.games.tbs.parameters.GameSettings;

import io.jenetics.Genotype;
import io.jenetics.IntegerChromosome;
import io.jenetics.IntegerGene;
import io.jenetics.engine.Codec;
import io.jenetics.util.Factory;

public class GeneDecoder implements Codec<GameSettings, IntegerGene> {
	private ParameterList parameters;
	private GameSettings common;
	private SettingsIO io;
	
	public GeneDecoder(ParameterList parameters, GameSettings common, SettingsIO io) {
		this.parameters = parameters;
		this.common = common;
		this.io = io;
	}
	
	
	@Override
	public Factory<Genotype<IntegerGene>> encoding() {

		return () -> {
				List<IntegerChromosome> chromosome = new ArrayList<>();	
				
				for (Parameter p : parameters) {
					int range = (p.getMax() - p.getMin()) / p.getStep();

					chromosome.add(IntegerChromosome.of(0, range, 1));
				}
				
				return Genotype.of(chromosome);
		};
	}

	@Override
	public Function<Genotype<IntegerGene>, GameSettings> decoder() {
		return gene -> {
			GameSettings settings = parameters.apply(common, gene);
			settings.finish(io);
			return settings;
		};
	}

}
